package com.supervision.police.dto;

import com.supervision.police.domain.ModelAtomicIndex;
import com.supervision.police.domain.ModelAtomicResult;
import com.supervision.police.domain.ModelIndex;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import java.util.Objects;

/**
 * 案件原子指标DTO
 */
@Data
public class CaseAtomicIndexDTO {

    @Schema(description = "原子评估结果id")
    private String atomicResultId;

    @Schema(description = "案件id")
    private String caseId;

    @Schema(description = "原子指标id")
    private String atomicIndexId;

    @Schema(description = "原子指标名")
    private String automicIndexName;

    @Schema(description = "指标id")
    private String indexId;

    @Schema(description = "指标名")
    private String indexName;

    @Schema(description = "案件类型")
    private String caseType;

    @Schema(description = "指标来源 1:人工定义 2:数据库查询 3:图谱生成 4:大模型")
    private String indexSource;

    @Schema(description = "评估结果")
    private String atomicResult;

    @Schema(description = "备注")
    private String remark;

    public CaseAtomicIndexDTO() {
    }

    public CaseAtomicIndexDTO(ModelAtomicIndex modelAtomicIndex, ModelIndex modelIndex, ModelAtomicResult modelAtomicResult) {
        if (Objects.nonNull(modelAtomicIndex)){
            this.atomicIndexId = modelAtomicIndex.getId();
            this.automicIndexName = modelAtomicIndex.getName();
        }
        if (Objects.nonNull(modelIndex)){
            this.caseType = modelIndex.getCaseType();
            this.indexName = modelIndex.getName();
            this.indexId  = modelIndex.getId();

        }
        if (Objects.nonNull(modelAtomicResult)){
            this.atomicResult = modelAtomicResult.getAtomicResult();
            this.atomicResultId = modelAtomicResult.getId();
            this.caseId = modelAtomicResult.getCaseId();
//       todo:     this.remark = modelAtomicResult.getRemark();
        }
    }
}
